import processing.serial.*;

//experimental rate viewer, (c) Loe Feijs TU/e 2015-2017
//use arduino and Geert Langereis' program to get beat-to-beat events
//see wolfram's mathworld for the equation of the cardioid.
//see Loe's slides of 21-2-2017 on http://wiki.id.tue.nl/s4ph/DBB170201609

Serial myPort; 
void setup() {
     size(900,900);
     background(0);
     blueEllipses();
     iniPoints(); 
     //if [0] does't work, try [1] or [2]:
     myPort = new Serial(this, Serial.list()[2], 19200);
}

float RR = 1000;           //beat to beat interval
float RRprev = 1000;       //previous value
float RRprevprev = 1000;   //previous previous
float RRavg = 1000;        //averaged
float RRavgprev = 1000;    //previous averaged

int beats = 0;
boolean flag = false;
int t10ms = 0;
void draw(){
     t10ms++;
     if (flag) {
         flag = false;

         float tau = 5;//beats
         float alpha = 1/tau;//weighing factor for smoothing
         RRavgprev = RRavg;
         RRavg = alpha * RR + (1 - alpha) * RRavg;//exponential smoothing
         
         stroke(100,0,0); fill(80,0,0); ellipse(width/2 + (RRprevprev - RRavgprev),height/2 - (RRprev - RRavgprev),6,6);//erase prev
         stroke(255,0,0); fill(255,0,0); ellipse(width/2 + (RRprev - RRavg),height/2 - (RR - RRavg),5,5);//poincaré.classic
         
         float x = t10ms/2.5;//x scaled time
         cardioid(x % width, 25, 6);
         if (x % width > width - 25) {
             //right edge is near, restart
             beats = 0;
             background(0);
             blueEllipses();
         }
         x = x % width;
         float y = (2000-RR) / 10;//y scaled by 10 
         addPoint(x, height - y);
         if (beats > 3) myCurve(myPoints);
     }   //end if  
}

void serialEvent(Serial p) { 
     if (p.available() > 0) {
         String val = myPort.readStringUntil('\n'); 
         if (val != null){
             print("3> ");
             beats++;
             flag = true;
             RRprevprev = RRprev;
             RRprev = RR;
             RR = unhex(trim(val)) / 1000;
             print("RR = " + RR + ", ");
             println("HR = " + ceil(60.0 / (0.01 + RRavg/1000)) + ", ");
         }
     }
}